# Generated by Django 3.2.19 on 2023-07-04 08:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0031_auto_20230615_0705'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterpriseoffer',
            name='amount_offer_spent_exec_ed',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='enterpriseoffer',
            name='amount_offer_spent_ocm',
            field=models.FloatField(null=True),
        ),
    ]
